package com.yixin.ms.server;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.google.common.base.Preconditions;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.crypto.util.SzhzSm4Util;
import com.yixin.ms.model.dto.IrsShortTermDTO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.dto.HealthCodeDTO;
import com.yixin.ms.server.vo.HealthCodeVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * @author: lileilei
 * @date: 2022/8/12 17:35
 * @description: 杭州市数据局根据barcode查询健康码（国密接口）信息
 */
@Slf4j
@Component
public class HealthServer {
    @Value("${health.appKey}")
    private String appKey;
    @Value("${health.appSecret}")
    private String appSecret;
    @Value("${health.queryInfoByBarCodeUrl}")
    private String queryInfoByBarCodeUrl;
    @Value("${health.getProvinceHealthCodeUrl}")
    private String getProvinceHealthCodeUrl;
    /**
     * 通过barcode查询个人信息
     * @param barcode
     * @return
     */
    public RestResult<HealthCodeVO> queryInfoByBarCode(String barcode){
        String body = null;
        try {
            body = HttpUtil.post(queryInfoByBarCodeUrl, JSON.toJSONString(new HealthCodeDTO().setAppKey(appKey).setAppSecret(appSecret).setBarCode(barcode)));
        }catch (Exception e){
            throw new ServiceException("barCode查询信息异常，请稍后重试");
        }

        RestResult restResult = JSON.parseObject(body, RestResult.class);
        if(restResult.getCode() == RestCode.SUCCESS.getCode()){
            HealthCodeVO healthCodeVO = JSON.parseObject(JSON.toJSONString(restResult.getData()), HealthCodeVO.class);
            restResult.setData(healthCodeVO);
        }

        return restResult;
    }
    /**
     * 获取省健康码信息
     *
     * @param idNumber
     * @return
     */
    public HealtCodeUserVO getProvinceHealthCodeInfo(String idNumber) {
        String body = null;
        try {
            if (StrUtil.isNotBlank(idNumber)) {
                body = HttpUtil.post(getProvinceHealthCodeUrl, JSON.toJSONString(new IrsShortTermDTO().setAppKey(appKey).setAppSecret(appSecret).setIdNumber(idNumber)));
                log.info("--->获取省健康码信息最终结果返回：{}", body);
                if (StrUtil.isNotBlank(body)) {
                    HealtCodeUserVO restResult = JSON.parseObject(body, HealtCodeUserVO.class);
                    return restResult;
                }
            }
        } catch (Exception e) {
//            throw new ServiceException("省健康码查询信息异常，请稍后重试");
        }
        return null;
    }
}
