package com.yixin.ms.server;

import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.AES;
import cn.hutool.http.HttpRequest;
import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.yixin.ms.boot.restful.RestCode;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.crypto.util.SzhzSm2Util;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.ColorEnum;
import com.yixin.ms.model.enums.HealthOpenEnum;
import com.yixin.ms.model.enums.ThirdPlatformPassIconEnum;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.dto.*;
import com.yixin.ms.server.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Arrays;
import java.util.Date;

/**
 * @author: lileilei
 * @date: 2022/6/23 10:47
 * @description: 杭州“四码一屏”统一接口
 */
@Component
@Slf4j
public class PlaceHealthCodeServer {
    private static final String TOKEN_KEY = "health:code:token";
    @Value("${health.code.clientId}")
    private String clientId;

    @Value("${health.code.clientSecret}")
    private String clientSecret;

    @Value("${health.code.publicKey}")
    private String publicKey;

    @Value("${health.code.getTokenUrl}")
    private String getTokenUrl;

    @Value("${health.code.deviceScanIdCardUrl}")
    private String deviceScanIdCardUrl;

    @Value("${health.code.deviceScanIdCardEncryptUrl}")
    private String deviceScanIdCardEncryptUrl;

    @Value("${health.code.deviceScanCodeUrl}")
    private String deviceScanCodeUrl;

    @Value("${health.code.queryPlaceInfoBySnUrl}")
    private String queryPlaceInfoBySnUrl;

    @Value("${health.code.savePlaceRecordUrl}")
    private String savePlaceRecordUrl;

    @Value("${health.code.queryUserExamInfo}")
    private String queryUserExamInfo;

    @Autowired
    private RestTemplate restTemplate;
    @Value("${health.code.passWhitelistUrl}")
    private String passWhitelistUrl;

    @Value("${health.code.deviceZoneUrl}")
    private String deviceZoneUrl;
    @Value("${health.code.whitelistUserInfoUrl}")
    private String whitelistUserInfoUrl;

    @Value("${health.code.whitelistUserQueryOpen}")
    private String whitelistUserQueryOpen;

    @Value("${health.code.qrcodePassQueryOpen}")
    private String qrcodePassQueryOpen;
    /**
     * 获取token
     *
     * @return
     */
    public HealthResultVO<TokenDataDTO> getToken() {
        HealthResultVO healthResultVO = restTemplate.getForObject(getTokenUrl, HealthResultVO.class);
        log.info("获取token响应内容：{}", JSON.toJSONString(healthResultVO));
        healthResultVO.setData(JSON.parseObject(JSON.toJSONString(healthResultVO.getData()), TokenDataDTO.class));
        return healthResultVO;
    }

    /**
     * 通过设备sn查询场所码信息
     */
    public RestResult<PlaceInfoVO> queryPlaceInfoBySn(String sn) {
        RestResult result = null;
        try {
            result = restTemplate.getForObject(new StringBuilder().append(queryPlaceInfoBySnUrl).append("?sn=").append(sn).toString(), RestResult.class);
            log.info("通过设备sn: {}, 查询账号信息返回内容:{}", sn, JSON.toJSONString(result));
        } catch (Exception e) {
            log.error("通过设备sn查询场所码信息异常：{}", e.getMessage());
            e.printStackTrace();
            return result;
        }
        if (result.getCode() == RestCode.DEVICE_NOT_FOUND.getCode()) {
            //设备未注册 未录入系统
            throw new ServiceException(RestCode.DEVICE_NOT_FOUND.getMsg());
        }
        result.setData(JSON.parseObject(JSON.toJSONString(result.getData()), PlaceInfoVO.class));
        return result;
    }


    public RestResult<QueryExamUserWhitelistVO> getExamUserWhitelist(String sn, String idNumber) {
        RestResult result = null;
        try {
            JSONObject object = new JSONObject();
            object.put("sn", sn);
            object.put("idNumber", idNumber);
            result = restTemplate.postForObject(queryUserExamInfo, object, RestResult.class);
            log.info("通过sn:{},idNumber:{},查询用户考试信息返回内容:{}", sn, idNumber, JSON.toJSONString(result));
            result.setData(JSON.parseObject(JSON.toJSONString(result.getData()), QueryExamUserWhitelistVO.class));
        } catch (Exception e) {
            log.error("查询用户考试信息异常：{}", e.getMessage());
            e.printStackTrace();
        }
        return result;
    }

    private void initApartmentInfo(HealtCodeUserVO userVO) {
        Integer isOpen = HealthOpenEnum.RELEASE.getCode();
        String codeTypeVOContent = "请通行";
        //准考证号
        String travelValidationTitle = "请通行";
        String travelValidation = " ";
        String vtitle = "通行";

        String iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/trip/green.png";
        //开门信号
        userVO.setIsOpen(isOpen);
        //语音播报
        userVO.setOpenMsg(codeTypeVOContent);
        userVO.setOpenVoiceMsg(codeTypeVOContent);
        //全局背景
        userVO.setBackgroundColor(ColorEnum.GREEN.getValue());
        //头部
        String title = "温馨提醒"; //温馨提醒
        userVO.setTitle(title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());
        //中上部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt(StrUtil.EMPTY);
        codeTypeVO.setContent(codeTypeVOContent);
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));
        //中下部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        //左icon
        v.setIconUrl(iconUrl);
        v.setTitle(vtitle);
        v.setContent("").setContentColor(ColorEnum.GREEN.getValue());
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (Exception e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));

        //右icon
        userVO.setVaccineIconUrl(vaccineIconUrl);
        userVO.setVaccineIcoTitle(travelValidation);
        userVO.setVaccineNum("");
        userVO.setVaccineNumColor(ColorEnum.GREEN.getValue());

        //底部
        userVO.setTravelValidationIconUrl(travelValidationIconUrl);
        userVO.setTravelValidationTitle(travelValidationTitle);
        userVO.setTravelValidation(travelValidation);
        userVO.setTravelValidationDesc(StrUtil.EMPTY);



        userVO.setASubLineFirst(StrUtil.isBlank(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        userVO.setASubLineSecond(StrUtil.isBlank(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setAMainIcon(StrUtil.isNotBlank(userVO.getAMainIcon()) ? userVO.getAMainIcon() : ThirdPlatformPassIconEnum.APASS.getCode()+"");
        userVO.setAMainLine(StrUtil.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : codeTypeVOContent);
        log.info("公寓返回数据:{}",JSONObject.toJSONString(userVO));

    }


    private void checkZoneDevice(RestResult dresult) {
        if (dresult == null) {
            throw new ServiceException(701, "设备未绑定区域");
        }
        ZoneDeviceVo zoneDeviceVo = JSON.parseObject(JSON.toJSONString(dresult.getData()), ZoneDeviceVo.class);
        if (zoneDeviceVo == null) {
            throw new ServiceException(701, "设备未绑定区域");
        }
        if (zoneDeviceVo.getLh() == null) {
            throw new ServiceException(701, "设备区域楼号信息不完整");
        }
        if (zoneDeviceVo.getSchoolBuildingCode() == null) {
            throw new ServiceException(701, "学校楼宇代码信息不完整");
        }

    }

    public RestResult schoolApartment2(ShortTermDTO dto, String idNumber, PlaceInfoVO placeInfoVO) {
        if (StrUtil.isEmpty(idNumber) && StrUtil.isBlank(dto.getHealthCode())) {
            throw new ServiceException(701, "请刷脸或扫描身份证或刷码进入");
        }
        if (StrUtil.isEmpty(placeInfoVO.getSchoolCode())) {
            throw new ServiceException(701, "学校代码配置不全，请联系系统管理员");
        }
        if (StrUtil.isEmpty(placeInfoVO.getSchoolCampusCode())) {
            throw new ServiceException(701, "区域代码配置不全，请联系系统管理员");

        }
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        if(StrUtil.isNotBlank(dto.getHealthCode())){
            return passByHealthCode(dto,userVO);
        }else {
            return passByIdNumber(dto, idNumber, userVO);
        }
    }

    private RestResult<HealtCodeUserVO> passByHealthCode(ShortTermDTO dto, HealtCodeUserVO userVO) {
        QrcodePassSearch search = new QrcodePassSearch();
        search.setSn(dto.getSn());
        search.setQrcode(dto.getHealthCode());

        RestResult<QrcodePassVo> request = HttpRequest.post(qrcodePassQueryOpen)
                .body(JSONObject.toJSONString(search)).timeout(4000).thenFunction(val -> {
                    String body = val.body();
                    log.info("设备号:{},请求三方返回打印:{}", dto.getSn(), body);
                    return JSON.parseObject(body, new TypeReference<RestResult<QrcodePassVo>>() {
                    });
                });
        if(200 != request.getCode()){
            throw new ServiceException(701, request.getMsg());
        }
        QrcodePassVo data = request.getData();
        if(data != null){
            userVO.setIdNumber(StrUtil.isEmpty(data.getIdNumber()) ? StrUtil.EMPTY : data.getIdNumber());
            userVO.setShowIdNumber(StrUtil.isEmpty(data.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(data.getIdNumber(), 4, 4));
            userVO.setUsername(StrUtil.isEmpty(data.getName()) ? StrUtil.EMPTY : data.getName());
            userVO.setShowUsername(StrUtil.isEmpty(data.getName()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(data.getName()));
        }
        initApartmentInfo(userVO);
        return RestResponse.ok(userVO);
    }

    private RestResult<HealtCodeUserVO> passByIdNumber(ShortTermDTO dto, String idNumber, HealtCodeUserVO userVO) {
        WhitelistPassSearch search = new WhitelistPassSearch();
        search.setSn(dto.getSn());
        search.setIdNumber(idNumber);
        //获取设备所属区域信息
        RestResult dresult = null;
        //获取人员学工号信息
        RestResult wresult = null;
        //学生多次刷脸
        RestResult faceResult = null;
        try {
            //获取设备所属区域信息
            dresult = restTemplate.postForObject(deviceZoneUrl, search, RestResult.class);
            log.info("--->获取设备所属区域信息返回结果：{}", JSONObject.toJSONString(dresult));
            //获取人员学工号信息
            wresult = restTemplate.postForObject(whitelistUserInfoUrl, search, RestResult.class);
            log.info("--->人员学工号信息返回结果：{}", JSONObject.toJSONString(wresult));
            faceResult = restTemplate.postForObject(whitelistUserQueryOpen, search, RestResult.class);
            log.info("--->学生多次刷脸：{}", JSONObject.toJSONString(faceResult));
        } catch (Exception e) {
            e.printStackTrace();
        }
        checkZoneDevice(dresult);
        if(faceResult == null || faceResult.getCode() != 200){
            throw new ServiceException("重复刷脸禁止通行");
        }

        userVO.setIdNumber(StrUtil.isEmpty(dto.getIdNumber()) ? StrUtil.EMPTY : dto.getIdNumber());
        userVO.setShowIdNumber(StrUtil.isEmpty(dto.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(dto.getIdNumber(), 4, 4));
        userVO.setUsername(StrUtil.isEmpty(dto.getName()) ? StrUtil.EMPTY : dto.getName());
        userVO.setShowUsername(StrUtil.isEmpty(dto.getName()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(dto.getName()));
        initApartmentInfo(userVO);
        return RestResponse.ok(userVO);
    }
}
