package com.yixin.ms.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.DesensitizedUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Lists;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.DESUtil;
import com.yixin.ms.boot.uitls.OkHttpUtils;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.HealthCodeServer;
import com.yixin.ms.server.HealthServer;
import com.yixin.ms.server.JiliangUniversityServer;
import com.yixin.ms.server.PlaceHealthCodeServer;
import com.yixin.ms.server.dto.JiliangApartmentDTO;
import com.yixin.ms.server.dto.JiliangCanPassDataDTO;
import com.yixin.ms.server.vo.HealthCodeVO;
import com.yixin.ms.server.vo.JiliangCanPassVO;
import com.yixin.ms.server.vo.PlaceInfoVO;
import com.yixin.ms.server.vo.QueryExamUserWhitelistVO;
import com.yixin.ms.service.HealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;


@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private HealthCodeServer healthCodeServer;
    @Autowired
    private JiliangUniversityServer jiliangUniversityServer;
    @Autowired
    private HealthServer healthServer;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;

    @Value("${jiliang.bcyyCanPassAdminId}")
    private int bcyyCanPassAdminId;

    @Override
    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        try {
            //增加上层接口 省健康码基本信息、疫苗注射数据信息、核酸检测数据 自定义封装三合一接口
            HealtCodeUserVO shortTermUser = healthServer.getProvinceHealthCodeInfo(shortTermDTO.getIdNumber());
            if (shortTermUser == null) {
                shortTermUser = getShortTermUser(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            }
            if (shortTermUser != null && !CollectionUtils.isEmpty(shortTermUser.getData())) {
                return shortTermUser;
            }
            return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
        } catch (Exception e) {
            e.printStackTrace();
            log.info("异常 :" + e.getMessage());
            try {
                return getSjJkm(shortTermDTO.getIdNumber(), shortTermDTO.getName());
            } catch (Exception ex) {
                ex.printStackTrace();
                throw new ServiceException("未获取到健康码信息，请联系工作人员");
            }
        }

    }

    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }

    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    @Override
    public RestResult<HealtCodeUserVO> getShortTerm(ShortTermDTO shortTermDTO) {
        log.info("联动服务查询入参信息：{}", JSON.toJSONString(shortTermDTO));
        if (StringUtils.isBlank(shortTermDTO.getToken())) {
            throw new ServiceException(700, "token为空");
        }

        if (StrUtil.isEmpty(shortTermDTO.getHealthCode())
                && StringUtils.isBlank(shortTermDTO.getIdNumber()) && StringUtils.isBlank(shortTermDTO.getName())) {
            throw new ServiceException(701, "身份证和姓名为空");
        }

        JiliangCanPassDataDTO dto = new JiliangCanPassDataDTO();
        dto.setOpTime(DateUtil.date());
        dto.setSn(shortTermDTO.getSn());

        JiliangCanPassVO jiliangCanPassVO = null;
        RestResult<PlaceInfoVO> placeInfoVORestResult = placeHealthCodeServer.queryPlaceInfoBySn(shortTermDTO.getSn());
        if (placeInfoVORestResult == null || placeInfoVORestResult.getData() == null) {
            throw new ServiceException(701, "设备未录入");
        }

        //判断是否区域管控设备
        if (placeInfoVORestResult != null
                && placeInfoVORestResult.getData() != null
                && CollUtil.isNotEmpty(placeInfoVORestResult.getData().getPassDeviceList())
                && placeInfoVORestResult.getData().getPassDeviceList().contains(shortTermDTO.getSn())
                && placeInfoVORestResult.getData().getStudentManageSwitch() != null
                && placeInfoVORestResult.getData().getStudentManageSwitch()) {
            //判断是否接入学生公寓管理平台
            log.info("--->ENTER学生公寓管理平台");
            return placeHealthCodeServer.schoolApartment2(shortTermDTO, shortTermDTO.getIdNumber(), placeInfoVORestResult.getData());
        }

        //判断是否是考试设备
        if (placeInfoVORestResult != null
                && placeInfoVORestResult.getData() != null
                && placeInfoVORestResult.getData().getExamDevices() != null) {
            List<String> sns = Arrays.asList(placeInfoVORestResult.getData().getExamDevices().split(","));
            if (sns.contains(shortTermDTO.getSn())) {
                log.info("--->ENTER学生考试管理平台");
                return examBiz(shortTermDTO, placeInfoVORestResult.getData());
            }
        }

        //校车和计量大学开放平台
        if (placeInfoVORestResult != null
                && placeInfoVORestResult.getData() != null
                && placeInfoVORestResult.getData().getAdminId().intValue() == bcyyCanPassAdminId) {
            //计量大学校车系统平台
            convertBcyyCanPassDto(dto, shortTermDTO);
            jiliangCanPassVO = jiliangUniversityServer.bcyyCanPass(dto);
        } else {
            //计量大学开放平台
            convertCanPassDto(dto, shortTermDTO);
            jiliangCanPassVO = jiliangUniversityServer.canPass(dto);
        }

        if (jiliangCanPassVO != null && StrUtil.isNotEmpty(shortTermDTO.getHealthCode())) {
            shortTermDTO.setIdNumber(jiliangCanPassVO.getIdCard());
            shortTermDTO.setName(jiliangCanPassVO.getName());
        }

        if (placeInfoVORestResult != null
                && placeInfoVORestResult.getData() != null) {
            HealtCodeUserVO userVO = new HealtCodeUserVO();
            if (jiliangCanPassVO.getIdCard() != null) {
                userVO.setIdNumber(jiliangCanPassVO.getIdCard());
                userVO.setShowIdNumber(DesensitizedUtil.idCardNum(jiliangCanPassVO.getIdCard(), 4, 4));
            } else {
                userVO.setIdNumber(StrUtil.isEmpty(shortTermDTO.getIdNumber()) ? StrUtil.EMPTY : shortTermDTO.getIdNumber());
                userVO.setShowIdNumber(StrUtil.isEmpty(shortTermDTO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(shortTermDTO.getIdNumber(), 4, 4));
            }
            userVO.setUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? StrUtil.EMPTY : shortTermDTO.getName());
            userVO.setShowUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(shortTermDTO.getName()));
            initHealthInfo(userVO, placeInfoVORestResult.getData(), jiliangCanPassVO);
            return RestResponse.ok(userVO);
        }

        throw new ServiceException(703, "未查询到此数据");
    }

    //查询考试信息
    private RestResult examBiz(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        if (StrUtil.isBlankIfStr(shortTermDTO.getIdNumber())) {
            throw new ServiceException(701, "请刷脸或扫描身份证进入");
        }
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        userVO.setIdNumber(StrUtil.isEmpty(shortTermDTO.getIdNumber()) ? StrUtil.EMPTY : shortTermDTO.getIdNumber());
        userVO.setShowIdNumber(StrUtil.isEmpty(shortTermDTO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(shortTermDTO.getIdNumber(), 4, 4));
        userVO.setUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? StrUtil.EMPTY : shortTermDTO.getName());
        userVO.setShowUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(shortTermDTO.getName()));
        userVO.setShowQrcode(placeInfoVO.getShowQrcode());
        userVO.setAdvertise(placeInfoVO.getAdvertise());
        userVO.setQrcodeText(placeInfoVO.getQrcodeText());
        //查询
        RestResult<QueryExamUserWhitelistVO> result = placeHealthCodeServer.getExamUserWhitelist(shortTermDTO.getSn(), shortTermDTO.getIdNumber());
        initExamHealthInfo(userVO, placeInfoVO, result, shortTermDTO.getSn());
        return RestResponse.ok(userVO);
    }

    private void initExamHealthInfo(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, RestResult<QueryExamUserWhitelistVO> result, String sn) {
        Integer isOpen = HealthOpenEnum.RELEASE.getCode();
        String codeTypeVOContent = "刷脸成功";
        //准考证号
        String travelValidationTitle = "准考证号";
        String travelValidation = " ";
        String vtitle = "场次";

        //新生报道设备
        List<String> snslist = null;
        try {
            String key = "NEW_STUDENTS_CHECK_SN";
            if (redisUtil.hasKey(key)) {
                String sns = (String) redisUtil.get(key);
                snslist = Arrays.asList(sns.split(","));
                if (snslist.contains(sn)) {
                    codeTypeVOContent = "录取核验成功";
                    vtitle = " ";
                }
            }
        } catch (Exception e) {
            log.error("获取新生报到设备失败", e.getMessage());
            e.printStackTrace();
        }
        //场次
        String content = " ";
        //报考类别
        String vaccineNum = " ";
        if (result != null && result.getData() != null) {
            travelValidation = result.getData().getExamCode();
            content = result.getData().getExamSession();
            vaccineNum = result.getData().getExamType();
            if (snslist != null && snslist.size() > 0 && snslist.contains(sn)) {
                vtitle = content;
                content = " ";
            }
        }

        String iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/trip/green.png";
        if (result == null || result.getData() == null) {
            isOpen = HealthOpenEnum.NO_THOROUGHFARE.getCode();
            codeTypeVOContent = "未预约考试，禁止通行";
            if (snslist != null && snslist.size() > 0 && snslist.contains(sn)) {
                codeTypeVOContent = "无学生信息,请联系校方工作人员";
            }
            iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            userVO.setAMainIcon(ThirdPlatformPassIconEnum.AFORBID.getCode()+"");
        }


        //开门信号
        userVO.setIsOpen(isOpen);
        //语音播报
        userVO.setOpenMsg(codeTypeVOContent);
        userVO.setOpenVoiceMsg(codeTypeVOContent);

        //全局背景
        userVO.setBackgroundColor(ColorEnum.GREEN.getValue());

        //头部
        String title = "温馨提醒"; //温馨提醒
        if (placeInfoVO != null
                && placeInfoVO.getTopTitle() != null) {
            title = placeInfoVO.getTopTitle();
        }
        userVO.setTitle(title == null ? "温馨提醒" : title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());

        //中上部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt(StrUtil.EMPTY);
        codeTypeVO.setContent(codeTypeVOContent);
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        //中下部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        //左icon
        v.setIconUrl(iconUrl);
        v.setTitle(vtitle);
        v.setContent(content).setContentColor(ColorEnum.GREEN.getValue());
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (Exception e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));

        //右icon
        userVO.setVaccineIconUrl(vaccineIconUrl);
        userVO.setVaccineIcoTitle(travelValidation);
        userVO.setVaccineNum(vaccineNum);
        userVO.setVaccineNumColor(ColorEnum.GREEN.getValue());

        //底部
        userVO.setTravelValidationIconUrl(travelValidationIconUrl);
        userVO.setTravelValidationTitle(travelValidationTitle);
        userVO.setTravelValidation(travelValidation);
        userVO.setTravelValidationDesc(StrUtil.EMPTY);


        userVO.setASubLineFirst(StrUtil.isBlank(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        userVO.setASubLineSecond(StrUtil.isBlank(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setAMainIcon(StrUtil.isNotBlank(userVO.getAMainIcon()) ? userVO.getAMainIcon() : ThirdPlatformPassIconEnum.APASS.getCode()+"");
        userVO.setAMainLine(StrUtil.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : codeTypeVOContent);
        log.info("公寓返回数据:{}",JSONObject.toJSONString(userVO));
    }

    private void initHealthInfo(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, JiliangCanPassVO jiliangCanPassVO) {
        userVO.setShowQrcode(placeInfoVO.getShowQrcode());
        userVO.setAdvertise(placeInfoVO.getAdvertise());
        userVO.setQrcodeText(placeInfoVO.getQrcodeText());

        Integer isOpen = HealthOpenEnum.RELEASE.getCode();
        String codeTypeVOContent = "请通行";
        String iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/trip/green.png";
        ColorEnum colorEnum = ColorEnum.GREEN;
        if (CanPassEnum.NOT_THROUGH.getCode().equals(jiliangCanPassVO.getCanPass())){
            isOpen = HealthOpenEnum.NO_THOROUGHFARE.getCode();
            codeTypeVOContent = jiliangCanPassVO.getMessage();
            iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            colorEnum = ColorEnum.RED;
        }
        //开门信号
        userVO.setIsOpen(isOpen);
        //语音播报
        userVO.setOpenMsg(jiliangCanPassVO.getMessage() != null ? jiliangCanPassVO.getMessage() : "请通行");
        userVO.setOpenVoiceMsg(jiliangCanPassVO.getMessage() != null ? jiliangCanPassVO.getMessage() : "请通行");

        //全局背景
        userVO.setBackgroundColor(colorEnum.getValue());

        //头部
        String title = "温馨提醒"; //健康码 温馨提醒
        if (placeInfoVO != null
                && placeInfoVO.getTopTitle() != null) {
            title = placeInfoVO.getTopTitle();
        }
        userVO.setTitle(title == null ? "温馨提醒" : title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());

        //中上部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt(StrUtil.EMPTY);
        codeTypeVO.setContent(codeTypeVOContent);
        codeTypeVO.setContentColor(colorEnum.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        //中下部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        //左icon
        v.setIconUrl(iconUrl);
        v.setTitle("温馨提醒");
        v.setContent("佩戴口罩").setContentColor(colorEnum.getValue());
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (Exception e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));

        //右icon
        userVO.setVaccineIconUrl(vaccineIconUrl);
        userVO.setVaccineIcoTitle("温馨提醒");
        userVO.setVaccineNum("勤洗手");
        userVO.setVaccineNumColor(colorEnum.getValue());

        //底部
        userVO.setTravelValidationIconUrl(travelValidationIconUrl);
        userVO.setTravelValidationTitle(StrUtil.EMPTY);
        userVO.setTravelValidation(StrUtil.EMPTY);
        userVO.setTravelValidationDesc(StrUtil.EMPTY);

        userVO.setASubLineFirst(StrUtil.isBlank(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        userVO.setASubLineSecond(StrUtil.isBlank(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setAMainLine(StrUtil.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : codeTypeVOContent);

        this.extracted(userVO, placeInfoVO, codeTypeVOContent);
    }
    private static void extracted(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, String openMsg) {
        String openVoiceMsg;
        String subLine = openMsg;
        if (userVO.getIsOpen() != null
                && userVO.getIsOpen().intValue() == HealthOpenEnum.NO_THOROUGHFARE.getCode().intValue()) {//禁止通行
            //是否开启警报
            userVO.setIsAlarm(placeInfoVO.getIsAlarm());
            userVO.setVaccineIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/warning.png");
            userVO.setVaccineNumColor("#ed593b");
            openMsg = "禁止通行";
            subLine = "禁止通行";

            userVO.setOpenTimes(0);
            userVO.setAMainIcon(StrUtil.isNotBlank(userVO.getAMainIcon()) ? userVO.getAMainIcon() : ThirdPlatformPassIconEnum.AFORBID.getCode()+"");
            userVO.setAMainLine(StrUtil.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : openMsg);
        } else {//请通行

            userVO.setAMainIcon(StrUtil.isNotBlank(userVO.getAMainIcon()) ? userVO.getAMainIcon() : ThirdPlatformPassIconEnum.APASS.getCode()+"");
            userVO.setAMainLine(StrUtil.isNotBlank(userVO.getAMainLine()) ? userVO.getAMainLine() : openMsg);
        }

        //设置开门信号提示信息
        openVoiceMsg = openMsg;
        userVO.setOpenMsg(StrUtil.isNotBlank(subLine) ? subLine : openMsg);
        userVO.setOpenVoiceMsg(openVoiceMsg);
        //设置通行结果数据
        if(CollUtil.isNotEmpty(userVO.getData())){
            HealtCodeTypeVO codeTypeVO = userVO.getData().stream().findFirst().get();
            if (ObjectUtil.isNotEmpty(codeTypeVO)) {
                codeTypeVO.setContent(subLine);
                userVO.setData(Arrays.asList(codeTypeVO));
            }
        }
    }

    private void convertCanPassDto(JiliangCanPassDataDTO dto, ShortTermDTO shortTermDTO) {
        if (StrUtil.isNotEmpty(shortTermDTO.getIdNumber())) {
            dto.setType(QrCodeEnum.SFZ.getValue());
            dto.setIdCard(shortTermDTO.getIdNumber());
        } else {
            if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode())) {
                dto.setType(shortTermDTO.getHealthCode().startsWith(QrCodeEnum.FKM.getPrefixT()) ? QrCodeEnum.FKM.getValue() : QrCodeEnum.XYM.getValue());
                dto.setIdCard(shortTermDTO.getHealthCode());
            }
        }
    }

    private void convertBcyyCanPassDto(JiliangCanPassDataDTO dto, ShortTermDTO shortTermDTO) {
        dto.setRlsbsbbm(shortTermDTO.getSn());
        if (StrUtil.isNotEmpty(shortTermDTO.getIdNumber())) {
            dto.setType(QrCodeEnum.SFZ.getValue());
            dto.setBcyyid(QrCodeEnum.SFZ.getPrefixT() + shortTermDTO.getIdNumber());
        } else {
            if (StrUtil.isNotEmpty(shortTermDTO.getHealthCode())) {
                dto.setType(QrCodeEnum.YYM.getValue());
                dto.setBcyyid(shortTermDTO.getHealthCode());
            }
        }
    }

    @Override
    public RestResult<HealthCodeVO> queryInfoByBarCode(HealthCodeDTO healthCodeDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(healthCodeDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(healthCodeDTO.getAppSecret())) {
            return RestResponse.error("调用权限校验失败，请检查");
        }

        return healthCodeServer.queryInfoByBarCode(healthCodeDTO.getBarCode());
    }

    public HealtCodeUserVO getShortTermUser(String sfzh, String name) throws Exception {
        log.info("省大数据健康码传参，身份证{} 姓名 {}", sfzh, name);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("sfzh", sfzh);
        jsonObject.put("xm", name);
        String sync = OkHttpUtils.builder().url("http://115.236.38.205:8903/id/hvn")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(jsonObject.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("省大数据请重试");
        }
        log.info("省大数据健康码返回结果，{}", sync);
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.isNotBlank(sync)) {
            JSONObject json = JSON.parseObject(sync);
            if (StringUtils.equals("00", json.getString("error_code"))) {
                healtCodeUserVO.setData(JSON.parseArray(json.getJSONArray("data").toJSONString(), HealtCodeTypeVO.class));
                healtCodeUserVO.setNuclein(JSON.parseArray(json.getJSONArray("nuclein").toJSONString(), HealtCodeNucleinVO.class));
                //源数据只返回报告时间，检测时间先等同于报告时间，方便后面逻辑处理
//                healtCodeUserVO.getNuclein().forEach(v -> v.setReportTime(v.getChecktime()));
                healtCodeUserVO.setVaccineNum(json.getString("vaccine_num"));
            } else {
                throw new ServiceException("省大数据异常:" + jsonObject.getString("msg"));
            }
        }
        return healtCodeUserVO;
    }

    public HealtCodeUserVO getSjJkm(String sfzh, String name) throws Exception {
        log.info("市局健康码传参，身份证{} 姓名 {}", sfzh, name);
        String token = "";
        token = (String) redisUtil.get("jkm-token");
        if (StringUtils.isBlank(token)) {
            token = getToken();
            if (StringUtils.isBlank(token)) {
                throw new ServiceException("市局健康码请重试");
            }
        }
        JSONObject data = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("idCard", sfzh);
        String encryptSfzh = DESUtil.encrypt(body.toJSONString(), "7760ce65a8544f779f36607c201e8a0c");
        data.put("body", encryptSfzh);
        data.put("token", token);
        String sync = OkHttpUtils.builder().url("http://policecard.tenzont.com/auth/api/healthCode")
                .addHeader("Content-Type", "application/json; charset=utf-8")
                .localPost(data.toJSONString())
                .sync();
        if (sync.contains("请求失败：Connect timed out")) {
            throw new ServiceException("市局健康码请重试");
        }
        log.info("市局健康码返回结果，{}", sync);
        JSONObject jkm = JSONObject.parseObject(sync);
        String rspCode = jkm.getJSONObject("header").getString("rspCode");
        HealtCodeUserVO healtCodeUserVO = new HealtCodeUserVO();
        if (StringUtils.equals("00", rspCode)) {
            String mzt = jkm.getJSONObject("body").getString("mzt");
            if (StringUtils.isNotBlank(mzt)) {
                List<HealtCodeTypeVO> list = new ArrayList<>();
                HealtCodeTypeVO healtCodeTypeVO = new HealtCodeTypeVO();
                healtCodeTypeVO.setMzt(mzt);
                list.add(healtCodeTypeVO);
                healtCodeUserVO.setData(list);
                healtCodeUserVO.setNuclein(new ArrayList<>());
                healtCodeUserVO.setVaccineNum("");
                return healtCodeUserVO;
            } else {
                throw new ServiceException("市局健康码请重试");
            }
        } else if (StringUtils.equals("01", rspCode)) {
            getToken();
            HealtCodeUserVO sjJkm = getSjJkm(sfzh, name);
            return sjJkm;
        } else {
            throw new ServiceException("市局健康码请重试");
        }
    }


    public String getToken() throws Exception {
        String sync = "";
        JSONObject jsonObject = new JSONObject();
        JSONObject body = new JSONObject();
        body.put("appid", "jzz3c086d786420a958a143e6154a351");
        body.put("secret", "ecbe0e220ec74d05b2b22268a737d2f1");
        jsonObject.put("body", body);
        try {
            sync = OkHttpUtils
                    .builder()
                    .url("http://policecard.tenzont.com/auth/api/token")
                    .localPost(jsonObject.toJSONString())
                    .sync();
        } catch (Exception e) {
            log.info("获取健康码的token接口异常 ，{}", e.getMessage());
            return "";
        }
        if (StringUtils.isBlank(sync)) {
            return "";
        }
        JSONObject fhz = JSONObject.parseObject(sync);
        String rspCode = fhz.getJSONObject("header").getString("rspCode");
        if (StringUtils.equals("00", rspCode)) {
            String token = fhz.getJSONObject("body").getString("token");
            if (StringUtils.isNotBlank(token)) {
                redisUtil.set("jkm-token", token, 7200);
                return token;
            }
        }
        return "";
    }

//    public static void main(String[] args) throws Exception {
//        System.out.println(getSjJkm("412728199706030513", ""));
//    }
}

